/* gnumeric-features.h.  Generated from gnumeric-features.h.in by configure.  */
#ifndef _LIBSPREADSHEET_CONFIG_H_
#define _LIBSPREADSHEET_CONFIG_H_

/* Manually maintained to hold any compile time flags necessary to use the ABI
 * of libspreadsheet */

/* The Epoch of this release */
#define GNM_VERSION_EPOCH 1
/* Extra, possibly empty tag for this release */
#define GNM_VERSION_EXTRA ""
/* The Major version number of this release */
#define GNM_VERSION_MAJOR 12
/* The Minor version number of this release */
#define GNM_VERSION_MINOR 55
/* The version number of this release, with optional extra suffix */
#define GNM_VERSION_FULL "1.12.55"

/* Define if UI is built */
#define GNM_WITH_GTK 1
/* Define if the long double type is to be used */
/* #undef GNM_WITH_LONG_DOUBLE */

/* Define to 1 if Gnumeric supplies the `lgamma' function. */
/* #undef GNM_SUPPLIES_LGAMMA */

/* Define to 1 if Gnumeric supplies the `lgamma' function. */
/* #undef GNM_SUPPLIES_LGAMMA_R */

/* Define to 1 if Gnumeric supplies the `erfl' function. */
/* #undef GNM_SUPPLIES_ERFL */

/* Define to 1 if Gnumeric supplies the `erfcl' function. */
/* #undef GNM_SUPPLIES_ERFCL */

/* Define to 1 if Gnumeric reduces the argument range of sin/cos/tan */
/* #undef GNM_REDUCES_TRIG_RANGE */

#endif  /* _LIBSPREADSHEET_CONFIG_H_ */
